* NOTE1: MAKE SURE THAT STATA COMMANDS "outreg2" & "sdecode" are installed
* NOTE2: MAKE SURE THAT MACROS THAT COULD OVERLAP ARE DROPPED "macro drop _all" (if a macro is generated but variable has no observations, program will break)

* SECTION1: General Dataset Information

cap program drop globals
program def globals
	global path ZZZZ // replace ZZZ with directory
	global country Zzzz // country name with first letter capitalized (no space)
	global CCC ZZZ // WDI 3 letters country code (i.e. MEX, IDN)
	global data zzz // census if it is a census; survey if it is a survey
	global datatype zzz // cross if is a cross-section or several cross sections over time [same firms cannot be tracked over time]; panel if the same firms can be tracked over time; 
	global datasource "Source: ZZZ" // For example: "Bureau of Statistics, Establishments Listing" "Central Bank, Survey of Enterprises"
	global eps z // yes if eps graphs need to be generated; no otherwise
	global byyear = z // replace z with number of years in data
	global formality zzz // replace "zzz" with "tax" if data is from tax authorities, "registered" if data is from a Census / Survey, "physical location" if data may include informal/non-registered firms)
	global employment zzz // replace "zzz" with "permanent" if  permanent/paid employees; "total" total employment if  permanent not available; "both" if both available
	global sector zzz // replace "zzz" with "all" if the data covers ALL sectors; "except agriculture" if Agriculture is not covered; "except mining" is mining is not covered; "except agriculture and mining" if both sectors are not included; "manufacturing only" if only manufcuting is included
	global manuf = z //  Z = 1 if only manufacture; 0 if has also services
	global agric = z // Z = 1 if there are firms in agricultural Sector 
	global threshold = Z // replace "Z" = 1 if no restriction on size existed upon data creation; 10 if the data includes firms with at least 10 employees; 20 if the data includes firms with at least 20 employees
	global byyear = z // number of years with data information
	global startyr = zzzz
	global endyr = zzzz
	global report_age zzz // yes if age available; no otherwise
	global wgt zz    // Z = pw if probabilistic weights; Z =  fw if frequency weights; Z =aw if analytical weights or Z= fw if there are NO weights		
	global export = z //  Z = 1 if exports or whether firm export is reported; Z = 0 otherwise
	global ownership = z  // Z= 1 ownership available (i.e. domestic, foreign, state-owned); Z=0 otherwise
	global isicv4 = z // Z = number of digits of isicv4 once matched is done 
	global region = z // Z = 1 if variable region will be used in descriptive analysis; Z=0 otherwise
	*INSTRUCTIONS VARIABLE REGION: 
	* This variable is 1,2,...N and will be used for graphs; it may need "manual" aggregation of provinces
	*It should set 1 corresponding to the most important region (where the capital is, or the most urbanized)
	if $region == 1 {
		local k = 1
		local region "ZZZ ZZZ ZZZ ZZZ" // please, list the names of the regions in order matching the corresponding numbers (not more than 8 -this is used for graphs). PLEASE, assign region=1 to the most popular region
		foreach g in `region' {
				global lreg`k' "`g'"
				local k = `k' + 1
						}
		global mxreg = `k' - 1
					}
	global sales = Z   // replace "Z" with "1" if the data-series include sales; with "0" if sales information is not available
	global outputs = Z   // replace "Z" with "1" if the data-series include output (value of production before sales); with "0" if output information is not available
	global valueadded = Z   // replace "Z" with "1" if the data-series include value added (or sales and material inputs); with =="0" if value added information is not available
	global capital = Z   // replace "Z" with "1" if the data-series include capital stock or fixed assets measures; with =="0" if capital stock or fixed assets information is not available  
	global wages = z // 1 if the data-series reports only wage bill (excludes benefits); 2 if reports only total labor cost; 3 if reports both wage bill and total labor costs; 0 if it does not report either wages or labor cost
	global units = z // ZZZ =1 if values in units of local currency (constant across years); Z =  1000 if values ('000s) of local currency; Z =  1000000 if sales are reported in millions ('000 000s), etc..  
	if $units == 1 {
			global unit "Thousand LCU"
				}
	if $units == 1000 {
		global unit "Thousand LCU"
			}
	if $units == 1e6 {
		global unit "Million LCU"
			}
	if $units == 1e9 {
		global unit "Billion LCU"
			}
	
	* GO to SECTION 2
	
	global dofiles "$path\dofiles"
	global input "$path\input"
	global output "$path\output"
	global startyr2 = $startyr + 1
	global dataset "$input\\${CCC}_JDD.dta"
	global raw "$input\\${CCC}_raw.dta" // save your data in the input folder (after initial cleaning and with deflators and isicv4 code)
	global D = $isicv4
	
	if $valueadded == 1 {	
		global prodvar LPV
		global lnprodvartit "Value Added per Worker (log)"	
		global prodvartit "Value Added per Worker"	
		global Yvar rV
		global Yvartit "Value Added"
			}
	if $sales == 1  {	
			global Y rS
			global Ytit "Sales"	
			global Yvar rS
			global Yvartit "Sales"		
			if $valueadded == 0 {
				global prodvar LPQ
				global lnprodvartit "Sales per Worker (log)"	
				global prodvartit "Sales per Worker"	
								}
						}
	if $sales == 0 & $outputs == 1 {	
			global Y rY
			global Ytit "Output"	
			global Yvar rY
			global Yvartit "Output"	
			if $valueadded == 0 {
				global prodvar LPQ
				global lnprodvartit "Output per Worker (log)"	
				global prodvartit "Output per Worker"	
							}
						}
	if $sales == 0 & $outputs == 0  {	
		global Y ""
		global Yvar ""
		global lnprodvartit ""	
		global prodvartit ""			
			}
	if $wages ==  1 | $wages == 3 {
		global rLC "rpay"
		global rLCtit "Wage Bill"
		global rwagetit "Average Wage"
		global avwage "rwage"
				}
	if $wages ==  2 {
		global rLC "rLC"
		global rLCtit "Labor Cost"
		global avwage "rwage"
		global rwagetit "Average Wage"
				}
	if $wages ==  0 {
		global rLC ""
		global rLCtit ""
		global avwage ""
		global rwagetit ""
				}				
	global K ""
	if $capital == 1 {
		global K "rK"
		global rKtit "Capital"
			}	
	if $threshold == 1 & $manuf == 0 {
		global benchmark = 1
		global country1 Zzzz // replace Zzzz with country name from lifecycle dataset, for example: Afghanistan, Burkina Faso, Bangladesh etc.  
		global country2 Zzzz // replace Zzzz with country name from lifecycle dataset, for example: Afghanistan, Burkina Faso, Bangladesh etc. 
		global country3 Zzzz // replace Zzzz with country name from lifecycle dataset, for example: Afghanistan, Burkina Faso, Bangladesh etc.  
	
			}
	else {
		global benchmark = 0
			}
end

********************************************************************************
*SECTION 2: STANDARDIZE VARIABLES

* All variables "must" be generated (set to missing if not available)
* Deflators "must" be merged (see instructions below)
* Z = "name in original data" corresponding to new name (ren original name if varname already exist)
* If corresponding variable is defined differently, generate new variable by inserting appropriate code in the program below
* If dataset include several years, make sure codes and units are consistent
* Check response rate of variables. If response less than 50%, consider putting variable to missing (document)

cap program drop standardvar
program def standardvar
    * year
	cap g year = ZZZZ
	la var year "Year"

	* unique id
	g id = ZZZZ // replace Z with variable that identifies firm or create it
		la var id "Plant id -numeric"

	* weights
	g wgt = ZZZ // replace Z with variable that has weights; if there are NO weights replace Z with 1; be aware that frequency weights are integers
	la var wgt "Weight"

	* industry
	if $isicv4 == 4 {
		g PROD4= ZZZ
		g l = length(PROD4s)
		replace PROD4s = "0" + PROD4s if l == 3
		drop l
		g PROD3=substr(PROD4s,1,3)
		destring PROD3, replace
		g PROD2=substr(PROD4s,1,2)
		destring PROD2, replace
		drop PROD4s
				}
		*
	if $isicv4 == 3 {
		g PROD4= .
		g PROD3= .
		tostring PROD3, g(PROD3s)
		g l = length(PROD3s)
		replace PROD3s = "0" + PROD3s if l == 2
		drop l
		g PROD2=substr(PROD3s,1,2)
		destring PROD2, replace
		drop PROD3s
				}
		*
	if $isicv4 == 2 {
		g PROD4= .
		g PROD3= .
		g PROD2= .
				}
	*
	forvalues x=2/4 {
		la var PROD`x' "ISICv4 `x' digits"
				}
			
	* year start-up (when firm/establishment starts operations)
		cap g yr_start = ZZZZ
		replace yr_start = .
		drop if yr_start > $endyr & yr_start ~= .		
		la var yr_start "Year of startup -numeric"
		
		la var yr_start "Year of startup -numeric"
		
	* geographical location, e.g region/province or state/district/village, etc...
		
	g region = ZZZ // this variable is 1,2,...N and will be used for graphs; it may need "manual" aggregation of provinces
	la def region 0 ""
	forval i = 1(1)$mxreg {
		la def region `i' "${lreg`i'}", modify
			}
	la val region region

	g reg1 = (region == 1) if region ~= .
	
	g geoid = ZZZ
		la var geoid "province/state" // NOTE: this level will be used for regressions; Make sure that there is enough categories but not too many so that regressions wouldn't have power

	* ownership information: 
	g ownership = Z // 1=domestic;  2=foreign;  3=soe 
	la def ownership 1 "Domestic" 2 "Foreign" 3 "State Owned", modify
	la val ownership ownership
	la var ownership "ownership"
	g fdi = Z // dummy variable 1 if Foreign 0 otherwise
	la var fdi "Foreign"
	g dom = Z // dummy variable 1 if Domestic Private 0 otherwise
	la var dom "Domestic"
	g soe = Z // dummy variable 1 if State Owned 0 otherwise
	la var soe "State-owned"

	*units
	g unit = .
			if $units == 1 {
				replace unit = $units/1000
				global unit "Thousand LCU"
					}
		if $units == 1000 {
			replace unit = $units
			global unit "Thousand LCU"
				}
		if $units == 1e6 {
			replace unit = $units
			global unit "Million LCU"
				}
		if $units == 1e9 {
				replace unit = $units
				global unit "Billion LCU"
				}
		la var unit "$unit"	
			
	* total sales	
	g sales = zzzz*unit
		la var sales "Sales $unit (nominal )"
	 
	* output 
	g nq = zzz*unit  //gross value of production adjusted by change in finished goods (+ ending finished goods - beginning finished goods)      		
		la var nq "Output $unit (nominal )"
		
	* total exports
	g export_tot= zzz*unit
	la var export_tot "Total exports $unit (nominal )"    

	g exportsh =  100*export_tot/sales if sales~=. 	  // note: if export is reported as a % of sales write this variable instead	
		la var exportsh "% output exported"

	g exporter = (exportsh>0)  if exportsh ~= .
	la var exporter "Exporter"

	* total imports
	g import_tot= zzz*unit
	la var import_tot "Total imports $unit (nominal )"    

	* material inputs = cost incurred this year in intermediate materials ---if possible adjusted by change in material goods (+ beginning material goods - ending material goods)
	g nm =zzz*unit 
		la var nm "Materials $unit (nominal )"

	* services charges 
	g ns =zzz*unit
		la var ns "Services charges $unit (nominal )"
		
	* total fuels used
	g tot_fuels= zzz*unit
		la var tot_fuels "Total fuels $unit (nominal )"
		
	 * total inventories 
	g tot_inven =  zzz*unit
		la var tot_inven "Total inventories $unit (nominal )"
		
	 * change inventories (ending goods - beginning goods)
	g chg_inven =  zzz*unit
		la var tot_inven "Total inventories $unit (nominal )"
		
	* value added (make sure cost of production does not include any labor cost)
	* 1st choice: nv=reported va (Note: replace with 2nd choice if missing; check that there is comparability for similar firms)
	* 2nd choice: nv = sales-cost production (intermediate materials,cost-services, fuel,...) 
	* 3rd choice: nv= nq-(nm-ns-(change in inventories))
	g nv = zzzz*unit // 2nd choice: VA is reported  
		la var nv "Value added $unit (nominal)"

	* 2nd choice:
	/*
	foreach x in nm ns tot_fuels {
		g `x'_1=`x'*(-1)
					}
	egen nv=rsum(sales nm_1 ns_1 tot_fuels_1), m
	drop nm_1 ns_1 tot_fuels_1
	*/

	* 3rd choice:
	/*
	foreach x in nm ns chg_inven {
		g `x'_1=`x'*(-1)
			}

	egen nv=rsum(sales nm_1 ns_1 tot_inven_1), m
	drop nm_1 ns_1 tot_inven_1
	*/
		

	* net value fixed assets after depreciation and monetary correction (if reported, preferred measure)

	g nk = zzz*unit // DEFAULT
		la var nk  "Capital excl. land & build. $unit (nominal )"  
		
	 g k_dep = zzz*unit
		la var k_dep  "Annual depretiation of capital $unit (nominal )"  
		
	* total capital stocks (replacement values) -it includes land and buildings (and  lease capital), but not new purchases 
	g nk_lb = zzz*unit
		la var nk_lb "Capital excl. land & build. $unit (nominal )"		

	* nk is used as default		
		
	* total investment 
	g inv_tot = zzz*unit
	la var inv_tot "Total net investment $unit (nominal )"

	g inv_rd = zzz*unit
		la var inv_rd "Net Investments  in intangibles $unit (nominal )"
		
	g inv_k = zzz*unit // if investment in capital is not reported, use recalled values of K and K_dep if available: inv_k=nk-(l.nk-l.nkdep)
		la var inv_k "Investments in mach. and equip. $unit (nominal )"
		
	*******EMPLOYMENT INFORMATION   	*** ONLY PAID EMPLOYMENT ***  
	* total wages (does not include benefits)
	g pay=zzz*unit 
		la var pay "Total wages $unit (nominal )"

	* total labor costs 
	g LC=zzz*unit 
		la var LC "Total labor costs $unit (nominal )"

	g benefits= LC-pay
		la var benefits "Total employment benefits $unit (nominal )"
	 
	* total employment 
	if "$employment" == "total" {
		g L=zzz
		la var L "Total employment"
						}
	*permanent paid employment  => DEFAULT
	if "$employment" == "permanent" {
		g L=zzz 
		la var L "Permanent employment"
				}

	if "$employment" == "both" {
		g Ltot=zzz 
		la var Ltot "Total employment"
		g L=zzz // => DEFAULT
		la var L "Permanent employment" 
				}
	drop if L < $threshold
			
	* NOTE: if female/male workers include both permanent and temporary, please document
	g fememp = zzz
		la var fememp "Total permanent female employment"
	   
	g maleemp = zzz
		la var maleemp "Total permanent male employment"

	* temporay employment: fixed- term contract, in contrast to permanent work where there is no end-date
	g tempemp=zzz
	   la var tempemp "Total temporary employment"
		
	* production employment (manual workers or blue-collar workers permanent employees: engaged in fabrication, assembly, transportation, maintenance, ///
	*					janitorial and guard services, auxiliary production (such as power plants), and other services closely related to the above activities. ///
	*					working supervisors are generally included, while apprentices and other trainees are generally excluded. 

	* NOTE: if production/non-production workers include both permanent and temporary, please document
	g prodemp = zzz
	   la var prodemp "Total permanent production employment"

	* total non-production employment
	g nprodemp = L - prodemp		
		la var nprodemp "total permanent non-production employment"

	* total wages of production workers
	g prodpay =  zzz*unit 
		la var prodpay "Total wages production workers $unit (nominal )"   	
		
	* total wages of production workers
	g nprodpay =  (pay-prodpay)
		la var nprodpay "Total wages non-production workers $unit (nominal )"

	* total wages of female workers
	g fempay =  zzz*unit 		
		la var fempay "Total wages female workers $unit (nominal )"
		
	* total wages of male workers   	 
	g malepay =  zzz*unit 	
		la var malepay "Total wages male workers $unit (nominal )"
		
	g permpay =zzz*unit 
		la var permpay "Total wages of permanent workers unit (nominal )"

	g temppay =zzz*unit
		la var temppay "Total wages temporal workers  $unit (nominal )"
		
	for var sales nq expor* impor* nm ns tot_fuels nv nk nk_lb k_dep *inv* *pay* LC benefits L: replace X = . if X<0 // Make sure original variables do not go to missing
	
	*Make sure deflators (go_p, va_p, k_p, mat_p, wag_p, cpi) are in the data
end

qui globals
		
		


