set more off
			
capture program drop constantind
program def constantind

	egen constind_mnmode = mode(`1'), by(id) min
	egen constind_mxmode = mode(`1'), by(id) max

	g constindx = constind_mnmode if constind_mnmode == constind_mxmode 

	egen lconstindyr = max(year) if `1'~= . , by(id)
	g constindlyr = `1' if constind_mnmode ~= constind_mxmode & year == lconstindyr 
	egen constind_lastyr = max(constindlyr), by(id)
	replace constindx= constind_lastyr if (constind_mnmode ~= constind_mxmode) | constindx == .

	replace `1' = constindx if constindx ~= .
	
	drop *constind*
end

capture program drop pay
program def pay
	sum `2'
	local n`2'=r(N)
	if `n`2''>0 {
		capture g `2'orig=`2'
		capture g `3'orig=`3'
		tempvar n group_PROD2 count_PROD2 
		g `n'=1 if `1'~=.
		bysort PROD2: egen `count_PROD2'=sum(`n') 
		egen `group_PROD2'=group(PROD2) if `count_PROD2'>=20 & `count_PROD2'~=.
		sum `group_PROD2'
		local xf = r(max)			
		forvalues i = 1(1)`xf' {	
			sum `1' if  `group_PROD2'==`i'
			replace `2'=. if (`1'>(r(mean)+4*r(sd))|`1'<(r(mean)-4*r(sd))) &  `group_PROD2'==`i'
				}
		sum `1' 
		replace `2'=. if (`1'>(r(mean)+4*r(sd))|`1'<(r(mean)-4*r(sd))) & `count_PROD2' <20
		replace `3'=. if `2'==.
		drop `n' `group_PROD2' `count_PROD2'
			}
end


qui use "$raw", clear
standardvar
cap ren N Norig
g N= 1
egen nyr = group(year)

	* constant industry across years 
	if "$datatype" == "panel" {
		qui constantind PROD$isicv4
				}
		
	*Exports can be >100, producers and traders; we leave it as it is

	count if exportsh > 100 & exportsh ~=.
	
	*First/last year in data (fyear=lyear if cross section data)
	bysort id:egen fyear=min(year)       		
		la var fyear "first year in dataset"

	bysort id:egen lyear=max(year)       		
		la var lyear "last year in dataset"
		
	* Firm's year started operations
	g yr_startorig = yr_start  
	count if yr_start == .
	replace yr_start = 1900 if yr_start <=1900      // recode extreme values to 1900
	if "$datatype" == "panel" {
		by id: egen yrstart_m = mode(yr_start), max  
		count if yrstart_m > fyear  & fyear~=. & yrstart_m~=.
		capture drop min_yrstart   
		egen min_yrstart=min(yr_start), by(id)
		replace yr_start = yrstart_m if yr_start ~= yrstart_m & yrstart_m ~= .
		replace yr_start = min_yrstart  if yrstart_m> fyear & min_yrstart ~= . & yrstart_m ~= .
		replace yr_start = fyear if yr_start> fyear & yr_start~=. & fyear~=. & $startyr~=fyear
		replace yr_start = yr_start+1 if fyear==yr_start+1 & $startyr~=fyear
		replace yr_start = yr_start+2 if fyear==yr_start+2 & $startyr~=fyear
		replace yr_start=. if fyear==$startyr & yr_startorig==.
		drop *yrstart*
				}
			
		if "$report_age"== "yes" {
			g age = year - yr_start + 1 if yr_start <= fyear & fyear ~= .
			replace age = year - fyear +1 if fyear>$startyr & fyear ~= . & age==. 
				}
		if "$report_age"== "no" {
			g age= year - fyear +1 if fyear>$startyr & fyear ~= .
				}
	if "$datatype" == "panel" {
		g exit = (year == lyear &  lyear ~= $endyr) 
			}
	g entry = (age == 1) if age ~= .
	
	* ===================================
	*  PART II: DEFLATES AND CLEANS DATA
	* ===================================
	*deflates
	g rS = sales/go_p
		la var rS "Sales $unit"
	g rV = nv/va_p
		la var rV "Value Added $unit"
	g rY = nq/go_p
		la var rY "Output $unit"
	g rM = nm/mat_p
		la var rM "Materials $unit"
	g rK = nk/k_p
		la var rK "Capital excl. land and buildings $unit"
	g rK_lb = nk_lb/k_p
		la var rK_lb "Capital incl. land and buildings $unit"
	g rpay = pay/wag_p
		la var rpay "Wage bill $unit"
	g rLC = LC/wag_p
		la var rLC "Labor cost $unit"
	g rexport_tot = export_tot/go_p
		la var rexport_tot "Exports $unit"
	g rimport_tot= import_tot/mat_p	 
		la var rimport_tot "Imports$unit"
	g rinv_tot = inv_tot/k_p
		la var rinv "Net Investments $unit"
	g rinv_rd = inv_rd/k_p
		la var rinv_rd "Net Investments  in intangibles $unit"
	g rinv_k = inv_k/k_p
		la var rinv_k "Investments machinery and equipment $unit"

	g lnrM = log(rM+1e-6)
	g lnrY = log(rY+1e-6)
	g lnrS = log(rS+1e-6)
	g lnrV = log(rV+1e-6)	 
	g lnrK = log(rK+1e-6)
	g lnrK_lb = log(rK_lb+1e-6)
	g lnL  = log(L)
	g lnrLC = log(rLC+1e-6)
	g lnrpay  = log(rpa+1e-6y)
	if $wages ~= 0 {
		g rwage = $rLC/L 
			}
	else {
		g rwage = .
	  }
	g lnrwage = log(rwage+1e-6)
	quietly {
		foreach t in prod nprod temp fem male {
			g r`t'pay = `t'pay/wag_p
			g lnr`t'pay = log(r`t'pay)
			g `t'rwage = r`t'pay/`t'emp
			g ln`t'rwage = log(`t'rwage)	
				}
			}
	 g lnrexport = log(rexport_tot)
	 g lnrimport = log(rimport_tot)
	 g lnrinv_tot = log(rinv_tot)
	 g lnrinv_rd = log(rinv_rd)
	 g lnrinv_k = log(rinv_k)			
	 
	la var lnL "Employment  (log)"	
	la var lnrV "Value Added $unit (log)"	
	la var lnrK "Capital excl. land and buildings $unit (log)"	
	la var lnrK_lb "Capital incl. land & build. $unit (log)"	
	la var lnrM "Materials $unit (log)"	
	la var lnrLC "Labor Costs $unit (log)"	
	la var lnrpay  "Labor Costs  $unit (log)"
	la var rwage "Average wage $unit"
	la var lnrwage "Average wage $unit (log)"
	foreach t in prod nprod temp fem male {
		la var r`t'pay "Total `t' workers wages $unit"
		la var `t'rwage "Av. wage `t' workers $unit"
		la var lnr`t'pay  "Total `t' workers wages $unit (log)"
		la var ln`t'rwage "Av. wage `t' workers $unit (log)"
			}
	la var lnrexport "Exports $unit (log)"
	la var lnrimport "Imports $unit (log)"
	la var lnrinv_tot "Total Investments $unit (log)"
	la var lnrinv_rd "Intangibles investments $unit (log)"
	la var lnrinv_k "Capital investments $unit (log)"
	 
	 * Outliers

		if $sales == 1 | $outputs == 1  {		
			tempvar ratio
			g `ratio'=rM/$Y
			pay `ratio' rM lnrM
			drop `ratio'

			sum rK
			local nk=r(N)
			if `nk'>0 {
				g `ratio'=rK/L
				pay `ratio' rK lnrK
				drop `ratio'
				g `ratio'=rK/$Y
				pay `ratio' rK lnrK
				drop `ratio'
						}
					}

	if $wages ~= 0 {				
			pay rwage rpay lnrpay
			replace lnrwage=. if rwage==.
			foreach t in prod nprod temp fem male {
				pay `t'rwage r`t'pay lnr`t'pay
				replace ln`t'rwage=. if `t'rwage==. 
					}
				}
				
		
	 * Share female and female majority
	g sh_fem = fememp/L
	g femmaj=1 if fememp/L >=0.5 & fememp~=.
	replace femmaj=0 if fememp/L<0.5
	la var femmaj "Firm has 50 percent or more female employees"

	*share skilled workers
	g sh_skll = nprodemp/L
	la var sh_skll "Share skilled workers"

	*share temporary workers
	g sh_temp = tempemp/L
	la var sh_temp "Share temporary workers"
				
	* AGGREGATED VARIABLES
	sum age
	global mxage = r(max)
	recode age (1/5 = 1 "1-5") (6/9 = 2 "6-9") (10/$mxage = 3 "10+"), gen (age_cls3)
	recode age (1/5 = 1 "1-5") (6/9 = 2 "6-9") (10/19 = 3 "10-19") (20/$mxage = 4 "20+") , gen (age_cls4)
	recode age (1/4 =1 "<5") (5/9 =  2 "5-9") (10/14 =  3 "10-14") (15/19 =  4 "15-19") (20/24 =  5 "20-24") ///
	(25/29 = 6 "25-29") (30/34 =   7 "30-34") (35/39 =  8 "35-39") (40/$mxage =  9 "40+"), gen(agehk_cls9)  
	foreach v in age_cls3 age_cls4 agehk_cls9 {
		la var `v' "`v'"
			}
	sum L
	global mxL = r(max)		
	recode L (1/19 = 1 "1-19") (20/99 = 2 "20-99") (100/$mxL = 3 "100+"), gen(sz_cls3)  
	recode L (1/9 = 1 "1-9") (10/19 = 2 "10-19") (20/99 = 3 "20-99") (100/499 = 4 "100-499") ///
	(500/$mxL = 5 "500+"), gen(sz_cls5)
		foreach v in sz_cls3 sz_cls5  {
		la var `v' "`v'"
			}

	if $manuf == 0 {
	recode PROD2 (1/2 = 1 "AgricHuntForest") (3 = 2 "Fishing") (5/9 = 3 "MiningQuarrying")  (10/33 = 4 "Manufacturing") (35/39 = 5 "Utilities") ///
	(41/43 = 6 "Construction") (45/47 = 7 "WholesaleRetail") (49/53 = 8 "TransportStorageComm") (55/56 = 9 "HotelsRestaurants")  ///
	(58/63 = 10 "InformationComm") (64/66 = 11 "FinanceInsurance") (68 = 12 "RealState") (69/75 = 13 "Professional") ///
	(77/82 = 14 "Administrative") (84/84 = 15 "PublicAdmin") (85/85 = 16 "Education") (86/88 = 17 "Health") (90/93 = 18 "Entertainment") ///
	(94/96 = 19 "OtherServices") (97/98 = 20 "Households")  (99/99 = 21 "Extraterritorial"), g(PROD1)
	recode PROD2 (1/3 = 1 "Agriculture") (5/9 35/39= 2 "MiningQuarryingUtilities")  ///
	(10/11 = 3 "FoodBeverages") (13/15 = 4 "TextilesApparelLeather") (12 16/23 26/33= 6 "OtherManufacturing") ///
	(24/25 = 5 "Metals") (41/43 = 7 "Construction") (45/47 = 8 "WholesaleRetail") ///
	(49/53  58/63= 9 "TransportCommStore")  (55/56 64/84= 10 "OtherServices") (84/. = . ), g(sector10)
	recode PROD1 (1 2 = 1 "Agriculture") (3 5 = 2 "MiningUtilities") (4 = 3 "Manufacture")  (6 = 4 "Construction")  (7/14 = 5 "Services") (15/. = . ), g(sector5)
	recode sector5 (1  = 1 "Agriculture") (2/4 = 2 "Industry")  (5 = 3 "Services"), g(sector3)
				}
	if $manuf == 1 {
	recode PROD2  (10/12 = 1 "Food&Bev&Tobacco")  (13/15=2 "Textiles&Apparel&Leath") (16/18 = 3 "Wood&Paper&Printing") (19/20 = 4 "Petroleum&Pharm&Chemicals") /// 
	(22 = 5 "Plastics")  (23 = 6 "NonMetallic_Cement&Clay") (24 = 7 "BasicMetals") (28/30 = 8 "Machinery&Vehic&Transport") (31 = 9 "Furniture") /// 
	 (21 25/27 32/33  = 10 "OtherManufacturing") (34/. = . ), g(sector10)
	recode sector10  (1 = 1 "Food&Bev&Tobacco")  (2=2 "Textiles&Apparel&Leath")  (6 = 3 "NonMetallic_Cement&Clay") /// 
	(7 = 4 "BasicMetals")  (3/5 8/10 = 5 "OtherManufacturing") , g(sector5)
	recode sector5 (1  = 1 "Food&Bev&Tobacco") (2 = 2 "Textiles&Apparel&Leath")  (3/5 = 3 "OtherManufacturing"), g(sector3)
				}				
	foreach v in sector3 sector5 sector10 {
		la var `v' "`v'"
			}
				
	if ($sales == 1 | $outputs == 1) & $valueadded == 0 {	
		g LPQ =$Y/L
		g outl_LPQ=.
		sum id if $Y ~= . & L ~= .
		local N = r(N)
		if `N' > 0 {
			tempvar n gPROD2 nPROD2 
			egen `n' = tag(id)  if $Y ~= . & L ~= .
			bysort PROD2: egen `nPROD2'=sum(`n') 
			egen `gPROD2' = group(PROD2) if `nPROD2' >= 20 & `nPROD2' ~= .
			sum `gPROD2'
			local xf = r(max)			
			forvalues i=1(1)`xf' {
						sum LPQ if `gPROD2'==`i', de
						replace outl_LPQ = 1 if (LPQ <= r(p1) | (LPQ >= r(p99) & LPQ ~= .)) & `gPROD2'==`i' 
								}
			sum LPQ, de
			replace outl_LPQ=1 if (LPQ<=r(p1) | (LPQ >= r(p99) & LPQ~=.)) & `nPROD2'<20  
				
		*drop very large ratios
		g LPQorig = LPQ 
		g lnLPQ = log(LPQ)
		replace outl_LPQ = 1 if lnLPQ >16 | lnLPQ < -16
		replace lnLPQ = . if outl_LPQ == 1
		replace LPQ = . if outl_LPQ == 1 | lnLPQ >16 | lnLPQ < -16

		if $sales == 1 {
			la var LPQ "Sales per worker"
			la var lnLPQ "Sales per worker -log"
			la var outl_LPQ "Outliers sales per worker"
				}
		if $sales == 0 & $outputs == 1 {
			la var LPQ "Output per worker"
			la var lnLPQ "Output per worker -log"
			la var outl_LPQ "Outliers output per worker"
				}
			}
		}
		
	*LPV // Value Added per worker
		if $valueadded == 1 {
			g LPV = rV/L
			g outl_LPV = .
			sum id if rV ~= . & L ~= .
			local N = r(N)
			if `N' > 0 {
				tempvar n gPROD2 nPROD2 
				egen `n' = tag(id)  if rV ~= .& L ~= .
				bysort PROD2: egen `nPROD2' = sum(`n') 
				egen `gPROD2' = group(PROD2) if `nPROD2' >= 20 & `nPROD2' ~= .
				sum `gPROD2'
				local xf = r(max)			/*last sector*/
				forvalues i=1(1)`xf' {
							sum LPV if `gPROD2'==`i', de
							replace outl_LPV=1 if (LPV<=r(p1)|(LPV>=r(p99) & LPV~=.)) & `gPROD2'==`i'  
								}
				sum LPV, de
				replace outl_LPV = 1 if (LPV <= r(p1) | (LPV >= r(p99) & LPV ~= .)) & `nPROD2'< 20
					}
		*drop very large ratios
			g LPVorig = LPV 
			g lnLPV = log(LPV)
			replace outl_LPV = 1 if lnLPV >16 | lnLPV < -16
			replace lnLPV = . if outl_LPV == 1
			replace LPV = . if outl_LPV == 1 | lnLPV >16 | lnLPV < -16
			la var LPV "Value Added per worker"
			la var lnLPV "Value Added per worker -log"
				}
			
	foreach i in LPQ lnLPQ LPV lnLPV {
		capture g `i' = .
		capture g outl_`i' = .
				}

	cap drop `n' `gPROD2' `nPROD2'		

	g ${prodvar}_quartile=. if ${prodvar} == 0
	la var ${prodvar}_quartile "${prodvartit} quartiles"	
	
	if "$prodvar" ~= "" {
			sum year  if ${prodvar} ~= .
			local mnyr = r(min)
			local mxyr = r(max)
			forvalues y=`mnyr'(1)`mxyr' {
				 sum ${prodvar} if year==`y' & ${prodvar} ~= 0, de
				 replace ${prodvar}_quartile = 1 if ${prodvar}<r(p25) & ${prodvar}~=0 & ${prodvar}~=. & ${prodvar}_quartile==. & year == `y'
				 replace ${prodvar}_quartile = 2 if ${prodvar}>=r(p25) & ${prodvar}<r(p50) & ${prodvar}~=. & ${prodvar}_quartile==. & year == `y'
				 replace ${prodvar}_quartile = 3 if ${prodvar}>=r(p50) & ${prodvar}<r(p75) & ${prodvar}~=. & ${prodvar}_quartile==. & year == `y'
				 replace ${prodvar}_quartile = 4 if ${prodvar}>=r(p75) & ${prodvar}~=. & ${prodvar}_quartile==. & year == `y'
						}
				la def Pq 1 "Q1-low" 2 "Q2" 3 "Q3" 4 "Q4-high", modify
				la val ${prodvar}_quartile Pq
					}
					
	if "$datatype" == "panel" {
		sort id year
		g Lg = 2*(L-l.L)/(L+l.L) if L~=. & l.L ~= .
		replace Lg = 2 if age == 1
		g ${prodvar}g = 2*(${prodvar}-l.${prodvar})/(${prodvar}+l.${prodvar})
		replace ${prodvar}g  = 2 if age == 1
		g Wg = 2*(rwage-l.rwage)/(rwage+l.rwage)
		replace Wg = 2 if age == 1
					}
					
					
	* Saving data
	tab PROD2 if PROD2 >= 84
	drop if PROD2 >= 84
		
	cap drop __00*

	order id year PROD* sect* geo* region L sz* age* $prodvar* *emp *rY *rV *rK *rM *rS *rwage*

	save "$dataset", replace
