capture program drop regsave
program define regsave, rclass
	version 8.2
	syntax [anything] [using/] [, Tstat Pval ci Level(real $S_level) noSE CMDline autoid covar(string) detail(name min=1) double ADDLABel(string asis) addvar(string) table(string) coefmat(string) varmat(string) df(real -999) append replace saveold(integer -999)]
				
	* Hold onto using filename in case it gets reset by further syntax commands
	local hold_using `"`using'"'
	
	* Unabbreviate varnames if wildcards are specified (we can't use -varlist- above in -syntax- because some specified regressors may not actually be vars, e.g., _cons)
	foreach input in anything covar {
		if strpos(`"``input''"',"*")!=0 | strpos(`"``input''"',"~")!=0 | strpos(`"``input''"',"?")!=0 | strpos(`"``input''"',"-")!=0 | strpos(`"``input''"',".")!=0{

			tokenize `"``input''"'
			while `"`1'"' != "" {
				if strpos(`"`1'"',"*")!=0 | strpos(`"`1'"',"~")!=0 | strpos(`"`1'"',"?")!=0 | strpos(`"`1'"',"-")!=0 | strpos(`"`1'"',".")!=0 {
					local 0 `"`1'"'
					cap syntax varlist(ts)
					if !_rc local `input' : subinstr local `input' "`1'" "`varlist'"
				}
				macro shift
			}
			local using `"`hold_using'"'
		}
	}
	
	* Saveold options, if specified
	local save save
	local saveold_opts ""
	if "`saveold'"!="-999" {
		local save "saveold"
		local saveold_opts "version(`saveold')"
		
		* Reuse in regsave_tbl subcommand:
		local sv_old "saveold(`saveold')"
	}
	
	* Apply namelist parsing syntax to anything, correcting for ts and fv varnames.  (-syntax namelist- doesn't have a ts or fv option unfortunately)
	* orig_namelist holds unchanged names, so it includes time series operators etc.
	local orig_namelist `"`anything'"' 
	
	* Table suboptions
	if "`table'"!= "" {
						
		local 0 `"`table'"'
		local tbl_command `"`0'"'
		syntax namelist(max=1) [, order(string) format(string) PARENtheses(namelist max=4) BRACKets(namelist max=4) *]
		local table `"`namelist'"'
		
		local 0 `", `options'"'
		syntax [, ASTERisk(numlist descending integer min=0 max=3 >=0 <=100)]
		
		* Detect if the asterisk option was specified; set defaults if significance levels were not specified by user
		local aster_defaults "10 5 1"
		if "`asterisk'"!="" local asterisk "asterisk(`asterisk')"
		if strpos(`"`0'"',"aster")!=0  & `"`asterisk'"'=="" local asterisk "asterisk(`aster_defaults')"
		
		* Asterisk option not allowed with nose, no pval, no tstat
		if "`asterisk'"!="" & "`pval'`tstat'"=="" & "`se'"!="" {
			di as error "Asterisk option requires presence of standard errors, pvals, or tstats"
			exit 198					
		}		
	}
	
	local using `"`hold_using'"'

	**********************************
	* Error check option selections  *
	**********************************
	* File options
	if "`append'`replace'`sv_old'"!="" & "`using'"=="" {
		di as error "Must specify a filename with `append'`replace' `sv_old'"
		exit 198
	}
	
	* CI options
	if `level' < 10 | `level' > 99.99 {
		di as error "level() must be between 10 and 99.99 inclusive"
		exit 198
	}

	* Detail options
	if !inlist(`"`detail'"',"all","scalars","macros","") {
		di as error `"The only valid choices for the detail option are "all", "scalars", or "macros""'
		exit 198		
	}
	
	* dprobit has a special default for coefmat and varmat
	if `"`e(cmd)'"'=="dprobit" & "`coefmat'"=="" local coefmat "e(dfdx)"
	if `"`e(cmd)'"'=="dprobit" & "`varmat'"=="" local varmat "e(se_dfdx)"

	* mi estimate has a special default, if post suboption is not specified
	if `"`e(cmd)'"'=="mi estimate" & "`coefmat'"=="" local coefmat "e(b_mi)"
	if `"`e(cmd)'"'=="mi estimate" & "`varmat'"=="" local varmat "e(V_mi)"
	
	
	* coefmat and varmat defaults are e(b) and e(V)
	if "`coefmat'"=="" local coefmat "e(b)"
	if "`varmat'"=="" local varmat "e(V)"
	if `df' <= 0 local df = e(df_r)
	confirm matrix `coefmat'
	confirm matrix `varmat'
	
	* Order and format options
	if "`format'" != "" { 
		if strpos("`format'", "s")!=0 { 
			di as err "use numeric format in format() option" 
			exit 198 
		} 	
		capture di `format' 12345.67890
		if _rc {
			di as err "format() option invalid"
			exit 198
		}
	}	
	
	* Parentheses and bracket options
	foreach option in parentheses brackets {
		if "``option''"!="" {
			tokenize "``option''"
			while "`1'" != "" {
				if !inlist("`1'","coef","stderr","tstat","pval","ci_upper","ci_lower") {
					di as error "`1' is not a valid section for `option'"
					exit 198
				}
				macro shift
			}
		}
	}
	
	* set more off if not already
	local more_setting `c(more)'
	set more off
	
	
	**********************************
	* Store results			  		 *
	**********************************

	
	* Preserve if user is saving to a file
	if "`using'" != "" preserve

	* Grab point estimates
	tempname coef covars vars temp
	matrix `coef' = `coefmat'
	
	* Grab standard errors, account for cases where (1) no covars available and (2) se's are reported instead of vars
	matrix `vars' = `varmat'
	if rowsof(`vars')==1 {		
		if "`covar'"!="" {
			di as error `"covar() is not a valid option because `varmat' does not contain covariances."'
			exit 198		
		}
	}
	else {
		matrix `covars' = `varmat'
		matrix `vars' = vecdiag(`covars')
	}
	
	* Grab varnames. Check for the case where we have equation names (happens with, for example, the heckman command)
	local matnames : colfullnames `coef'
	if strpos(`"`matnames'"',":")!=0 local eqnames = 1

	* Error check that names supplied by the user are valid, but don't worry about eqnames here.  Note that matrices store "L1." as "L."; same with F and S.
	local matnames_nmchk : colnames `coef'
	local matnames_nmchk : subinstr local matnames_nmchk ":" "_", all
	local orig_namelist : subinstr local orig_namelist "L1." "L."
	local orig_namelist : subinstr local orig_namelist "F1." "F."
	local orig_namelist : subinstr local orig_namelist "S1." "S."
	
	foreach var in `orig_namelist' `covar' {
				
		local match = 0
		foreach nm of local matnames_nmchk {
			if "`var'"=="`nm'" {
				local match = 1
			}
		}
		if `match'==0 {
			di as error "No estimation results available for variable `var'"
			exit 111
		}
		
		if "`var'"=="_id" & "`autoid'"!="" {
			di as error "Cannot specify autoid option when also retrieving estimates for a variable called {it:_id}"
			exit 198			
		}
	}
	
	* Save estimation results into dataset, and take the square root of variances (if variances, not se's, are reported)
	drop _all
	label drop _all
	matrix `coef' = `coef''	
	matrix `vars' = `vars''	
	cap svmat `double' `coef'
	cap svmat `double' `vars'
	ren `vars'1 stderr
	ren `coef'1 coef
	if strpos("`varmat'","se")==0 qui replace stderr = sqrt(stderr) // I assume here that se matrices have "se" in their name
	
	* Gen varname variable
	qui gen var = ""
	local row = 1
	foreach nm of local matnames {
		qui replace var = "`nm'" in `row'
		local row = `row'+1
	}

	* Grab covariances, if specified
	foreach v of local covar {
		local num_rows = rowsof(`covars')
		cap matrix `temp' = `covars'[1..`num_rows',"`v'"]
		if _rc!= 0 {
			if _rc==111 di as error "`v' is not a valid regressor"
			exit _rc
		}
		local covar_nm : subinstr local v "." "", all
		cap svmat `double' `temp', names(`covar_nm')
		qui ren `covar_nm'1 covar_`covar_nm'

	}

	* Addvar option
	qui gen byte _keep=0
	if `"`addvar'"'!="" {
		
		tokenize `"`addvar'"', parse(",")
		while "`1'" != "" {

			if "`1'"=="," {
				di as error "Invalid syntax for addvar()"
				exit 198
			}
			
			* Increase obs by one
			local row_num = _N+1
			qui set obs `row_num'

			* Varname
			qui cap replace var = "`1'" in `row_num'
			if "`1'"!="," macro shift
			if "`1'"=="," macro shift
			
			* Coef
			qui cap replace coef = `1' in `row_num'
			if "`1'"!="," macro shift
			if "`1'"=="," macro shift
			
			* std error
			qui cap replace stderr = `1' in `row_num'
			if "`1'"!="," macro shift
			macro shift
			
			* Make sure observation is not dropped
			qui replace _keep = 1 in `row_num'
		}
	}

	* Keep only specified variables
	if "`orig_namelist'"!="" {
		foreach var of local orig_namelist {
			if "`eqnames'"=="" qui replace _keep=1 if var=="`var'"
			else qui replace _keep=1 if strpos(var,"`var'")!=0 // eqnames are stored as eqname:var
		}
		qui keep if _keep==1
	}
	qui drop _keep
	
	* tstat, p-vals, and ci if specified
	qui gen `double' tstat = coef/stderr
	if `level'<0 local cilevel = `c(level)'
	else local cilevel = `level'
	
	if "`pval'`ci'"!="" {
		if "`df'"=="." {
			qui gen `double' pval = 2*(1-normprob(abs(tstat)))
			qui gen `double' t_ci = invnorm(1-(1-`cilevel'/100)/2)
		}
		else {
			qui gen `double' pval = tprob(`df', abs(tstat))
			qui gen `double' t_ci = abs(invttail(`df', (1-`cilevel' /100)/2))
		}
	}
	if "`ci'"!="" {
		qui gen `double' ci_lower = coef - t_ci*stderr
		qui gen `double' ci_upper = coef + t_ci*stderr
	}
	cap drop t_ci
	if "`pval'"=="" cap drop pval
	if "`tstat'"=="" drop tstat
	if "`se'"!="" drop stderr
	else local stderr stderr	


	**********************************
	* Fill in remaining data		 *
	**********************************


	* N, R^2, and regression command
	if e(N)!=. qui gen N = e(N)
	if e(r2)!=. qui gen `double' r2 = e(r2)
	if `"`e(cmdline)'"'!="" & "`cmdline'"!="" {
		qui gen cmdline = `"`e(cmdline)'"'
		* Issue warning if macro gets truncated
		if length(`"`e(cmdline)'"')>244 di in yellow "Macro e(cmdline) truncated to 244 characters"		
	}
	
	* Detailed statistics
	if "`detail'"!="" {
		
		* Scalars
		if "`detail'"=="all" | "`detail'"=="scalars" {
			local scalar_vars : e(scalars)
			foreach v of local scalar_vars {
				capture confirm integer number `e(`v')'
				if _rc==0 qui cap gen int `v' = e(`v')
				else qui cap gen `double' `v' = e(`v')
			}
		}
		
		* Macros
		if "`detail'"=="all" | "`detail'"=="macros" {
			local macro_vars : e(macros)
			foreach v of local macro_vars {
				qui cap gen `v' = `"`e(`v')'"'				
				* Issue warning if macro gets truncated
				if length(`"`e(`v')'"')>244 di in yellow "Macro e(`v') truncated to 244 characters"
			}
		}
	}
	
	* autoid
	local autoid_default = -99987
	if "`autoid'"!="" {
		if "`append'" == "" qui gen _id = 1
		else qui gen _id = `autoid_default'
	}
	
	
	* Data are allnumeric by default unless parentheses etc. are specified. Also, allnumeric is turned off in the label loop below if at least one of the labels are string.
	local allnumeric allnumeric
	if "`detail'`parentheses'`brackets'`cmdline'`asterisk'"!="" local allnumeric

	* Label option
	tokenize `"`addlabel'"', parse(",")
	local i = 1
	while "``i''" != "" {
		
		* Certain labelnames are not allowed
		if inlist(`"``i''"',"if","in") {
			di as error `"``i'' is an invalid name for a label"'
			exit 198
		}
		
		* Get varname, fix Stata 8 compatibility issue
		if substr("`c(version)'",1,2)=="8." confirm new variable ``i''
		else confirm new variable ``i'', exact
		cap confirm name ``i''
		if _rc {
			di as error "Invalid variable name specified for addlabel()"
			exit 198
		}
		local varname = "``i''"
		
		* Advance forward two spots, so index points to data contents.
		if "``i''"!="," local i = `i'+1
		if "``i''"=="," local i = `i'+1
		
		* Get data and generate new var. If label is '.' make it a number
		capture confirm number ``i''
		if _rc==0 | `"`=trim("``i''")'"'=="." qui gen `double' `varname' = ``i''
		else {
			* If user inputted a blank, ``i'' holds a comma, so fix that
			if "``i''"=="," qui gen `varname' = ""
			else qui gen `varname' = "``i''"
			local allnumeric
		}
		if "``i''"!="," local i = `i'+1
		local i = `i'+1
	}

	qui compress
	
	* Convert data to table format, if specified
	if "`table'"!="" {
				
		* Grab passthru values (asterisk option was handled separately up above)
		local 0 `"`tbl_command'"'
		syntax namelist(max=1) [, order(passthru) format(passthru) PARENtheses(passthru) BRACKets(passthru) *]
		if `"`hold_using'"'!= "" local using `"using `hold_using'"'
		
		* Create table		
		regsave_tbl `using', name("`table'") `order' `format' `parentheses' `brackets' `asterisk' `allnumeric' `append' `replace' df(`df') `autoid' `sv_old'
		
		* Restore using val (it is reset by the syntax command above)
		local using `"`hold_using'"'		
	}
	

	**********************************
	* Label vars and clean up		 *
	**********************************


	* Label vars
	cap label var var "Variable"
	cap label var coef "Coefficient"
	cap label var stderr "Standard error"
	cap label var tstat "t-statistic"
	cap label var pval "Two-tailed p-value"
	cap label var N "Number of observations"
	cap label var r2 "R-squared"
	cap label var r2_a "Adjusted R-squared"
	cap label var cmdline "Estimation command code"
	cap label var df_m "Model degrees of freedom"
	cap label var df_r "Residual degrees of freedom"
	cap label var F "F statistic"
	cap label var rmse "Root mean squared error"
	cap label var rss "Residual sum of squares"
	cap label var mss "Model sum of squares"
	cap label var ll "Log likelihood"
	cap label var ll_0 "Log likelihood, constant only model"
	cap label var model "Model name"
	cap label var vce "Variance-covariance estimation type"
	cap label var N_clust "Number of clusters"
	cap label var chi2 "Chi-squared"
	cap label var r2_p "Pseudo-R-squared"
	cap label var N_cdf "Number of completely determined successes"
	cap label var N_cds "Number of completely determined failures"
	cap label var chi2type "Type of model chi-squared test"
	cap label var ci_lower "`level'% confidence interval (lower bound)"
	cap label var ci_upper "`level'% confidence interval (upper bound)"

	* Save results if specified (regsave_tbl has already saved results if that code was run, so we need to only take care of non-table situations)
	cap order var
	qui compress
	if "`using'"!= "" & "`table'"=="" {
		if "`append'"!= "" {
				append using "`using'"
				local replace replace
				
				* Generate autoid identifier
				if "`autoid'"!="" {
					qui summ _id
					if `r(max)'< 0 qui replace _id = 1 if _id == `autoid_default'
					else qui replace _id = `r(max)'+1 if _id == `autoid_default'
				}
		}
		`save' "`using'", `replace' `saveold_opts'
	}
	
	* Return shape of new dataset
	return clear
	qui count
	return scalar N = `r(N)'
	unab all_vars : *
	local num_vars : word count `all_vars'
	return scalar k = `num_vars'	
	
	* Restore data, and set more back on if it was on before
	if "`using'" != "" restore
	set more `more_setting'

end

********************************************************************************
*TO CREATE TABLE
capture program drop regsave_tbl
program define regsave_tbl, rclass
	version 8.2

	syntax [varlist] [using/] [if] [in], name(name) [order(string) format(string) PARENtheses(namelist max=6) BRACKets(namelist max=6) allnumeric ASTERisk(numlist descending integer min=0 max=3 >=0 <=100) df(string) autoid append replace saveold(integer -999)]
		
	**********************************
	* Error check option selections  *
	**********************************
	* File options
	if "`append'`replace'"!="" & "`using'"=="" {
		di as error "Must specify a filename with `append'`replace'"
		exit 198
	}
	
	* Ensure there is data
	qui count `if' `in'
	if `r(N)'<1 {
		di as error "No observations"
		exit 198
	}
	
	* Make sure we don't have duplicate varnames (will cause errors during the reshape)
	qui duplicates report var `if' `in'
	if `r(unique_value)'!=`r(N)' {
		di as error "Duplicate observations in var detected.  This is not allowed."
		duplicates list var `if' `in', table divider sepby(var)
		exit 198
	}
	
	* Saveold options, if specified
	local save save
	local saveold_opts ""
	if "`saveold'"!="-999" {
		local save "saveold"
		local saveold_opts "version(`saveold')"
		
		* Reuse in regsave_tbl subcommand:
		local sv_old "saveold(`saveold')"
	}
	
	preserve
	
	* If df is blank need to set it to missing
	if `"`df'"'=="" local df "."

	* Keep variables of interest, and declare table name. Note that `varlist' is * when regsave_tbl is called by regsave.
	keep `varlist'
	qui cap keep `if' `in'
	local table "`name'"
	
	******
	* Separate constant/label variables from regression statisticis
	******
	* Regression vars are var, coef, stderr, pval, tstat, and covar_*
	
	* Identify constant variables (these come from, e.g., addlabel() option)
	tempfile cons_vars
	tempname id
	unab varnames : *
	foreach v of local varnames {
		
		* Skip the vars var, coef, stderr, pval, tstat, and anything starting with covar
		if inlist("`v'","var","coef","stderr","pval", "tstat", "ci_lower", "ci_upper") continue
		if substr("`v'",1,6) == "covar_" continue
		
		* Make sure remaining var is a constant
		qui tab `v'
		if `r(r)'>1 {
			di as error "`v' is not constant within the specified range of data"
			exit 198			
		}
		
		* Store variable name in a list
		local cons_varnames "`cons_varnames' `v'"		
	}
	
	* Save out the constant variables (will be remerged back on after the reshape)
	if "`cons_varnames'"!= "" {
		keep `cons_varnames'
		qui keep in 1
		qui gen byte `id' = 1
		sort `id'
		qui `save' "`cons_vars'", replace `saveold_opts'
	}
	restore, preserve
	
	* Keep variables of interest. Drop the constant variables.
	keep `varlist'
	qui cap keep `if' `in'	
	cap drop `cons_varnames'	


	***********************************
	* Make table for regression stats *
	***********************************

	* Ensure there are no duplicates in var
	qui duplicates report var
	if `r(unique_value)'!=_N {
		di as error "Nonunique values present in variable var"
		exit 198
	}
	
	* Rename the vars so we don't have any problems during reshape. Keep old names in a list so we can rename them back later.
	forval x = 1/`=_N' {
		tempname rename_`x'
		local nm = var[`x']
		local originals `"`originals' "`nm'" "'
		qui replace var = "`rename_`x''" in `x'
	}

	* Asterisk option: *-**-*** for 10% / 5% / 1% significance is the default
	if "`asterisk'"!="" {
		qui gen coef2 = coef // used only in one case below
		qui tostring coef, replace force format(`format')
		local allnumeric // allnumeric must be blank with asterisks
		
		* Grab the significance levels. Set them to missing if they were not specified.
		forval x=1/3 {
			tokenize "`asterisk'"
			cap local pval`x' = ``x''/100
			if "`pval`x''"=="" local pval`x' = .
		}

		* Try pval first, then tstat, then se
		cap d pval
		if _rc==0 {
			qui replace coef = coef + "*" if pval<=`pval1' & pval>max(-1,`pval2')
			if "`pval2'"!="." qui replace coef = coef + "**" if pval<=`pval2' & pval>max(-1,`pval3')
			if "`pval3'"!="." qui replace coef = coef + "***" if pval<=`pval3'
		}
		
		else {
			cap d tstat
			if _rc==0 {
				if "`df'"=="." {
					qui gen `double' pval = 2*(1-normprob(abs(tstat)))
				}
				else {
					qui gen `double' pval = tprob(`df', abs(tstat))
				}
				qui replace coef = coef + "*" if pval<=`pval1' & pval>max(-1,`pval2')
				if "`pval2'"!="." qui replace coef = coef + "**" if pval<=`pval2' & pval>max(-1,`pval3')
				if "`pval3'"!="." qui replace coef = coef + "***" if pval<=`pval3'
				drop pval			
			}
		
			else {
				cap d stderr
				if _rc==0 {
					qui gen `double' tstat = coef2/stderr 
					if "`df'"=="." {
						qui gen `double' pval = 2*(1-normprob(abs(tstat)))
					}
					else {
						qui gen `double' pval = tprob(`df', abs(tstat))
					}
					qui replace coef = coef + "*" if pval<=`pval1' & pval>max(-1,`pval2')
					if "`pval2'"!="." qui replace coef = coef + "**" if pval<=`pval2' & pval>max(-1,`pval3')
					if "`pval3'"!="." qui replace coef = coef + "***" if pval<=`pval3'
					drop pval tstat
				}	
			}
		}
		drop coef2
		qui tostring *, replace force format(`format') // All variables need to be string now, in preparation for the reshape
	}
	
	* Identify variables to be reshaped
	cap unab covs: covar_*
	cap unab pval: pval
	cap unab tstat: tstat
	cap unab stderr: stderr
	cap unab cis: ci*	

	* Declare temp files and create identifier for reshape command
	tempfile tf1 tf2
	qui gen byte `id'=1
	
	* Set run_no and save to temp file
	local run_no = 0
	qui `save' "`tf2'", replace `saveold_opts'

	* Reshape the data to make the regressor variables into their own columns
	foreach v in `covs' `cis' `pval' `tstat' `stderr' coef {
		keep var `v' `id'
		ren `v' z
		qui reshape wide z, i(`id') j(var) string

		qui gen _stat = "`v'"
		if `run_no'==0 qui `save' "`tf1'", replace `saveold_opts'
		else {
			qui append using "`tf1'"
			qui `save' "`tf1'", replace `saveold_opts'
		}
		local run_no = 1
		qui use "`tf2'", clear
	}
	qui use "`tf1'", clear
	qui drop `id'

	***
	* Make one column long
	***
	qui d
	local num_vars = `r(k)'-1
	qui count	
	qui set obs `=`r(N)'*`num_vars''
	local row = 1	
	local v_index = 1
	unab varnames : *
	tempname tbl
	
	* Check if we are in all numeric format or not
	if "`allnumeric'"!="" qui gen `tbl' = .
	else {
		qui tostring `varnames', replace force format(`format')
		qui gen `tbl' = ""
	}

	qui gen var = ""
	foreach v of local varnames {
		if "`v'"=="_stat" continue
			
		local scount = 1
		qui count if _stat!=""
		forval x=1/`r(N)' {
			local s = _stat[`x']

			* 1) Store the statistic
			qui replace `tbl' = `v'[`scount'] in `row'


			* 2) Store (correct) name of the variable
			local vname : word `v_index' of `originals'
			qui replace var = "`vname'_`s'" in `row'

			* Apply parentheses and brackets options
			tokenize "`parentheses'"
			if inlist("`s'","`1'","`2'","`3'","`4'","`5'","`6'") qui replace `tbl' = "(" + `tbl' + ")" in `row'
			tokenize "`brackets'"
			if inlist("`s'","`1'","`2'","`3'","`4'","`5'","`6'") qui replace `tbl' = "[" + `tbl' + "]" in `row'

			local row = `row'+1
			local scount = `scount'+1
		}
		local v_index = `v_index'+1
	}
	qui keep var `tbl'
	ren `tbl' `table'

	* Store a list of all variable names, and format table if they are numeric
	qui levels var, local(variable_names)
	qui cap format `table' `format'

	****************************************
	* Convert constant stats to table form *
	****************************************
	
	* Merge in the constants
	if "`cons_varnames'"!= "" {
		qui gen byte `id' = 1
		sort `id'
		qui merge `id' using "`cons_vars'", uniqusing
		drop _merge `id'
	}
		
	unab varnames : *

	* Make vars into strings, if necessary
	if "`allnumeric'"=="" {
		
		* Determine which variables need formatting
		foreach v of local varnames {
			local value = `v'[1]
			cap confirm integer number `value'
			if _rc!= 0 local float_vars "`float_vars' `v'" 
		}
		qui cap tostring `float_vars', force replace format(`format')
		if "`format'"!="" local format "%15.0fc"
		qui tostring *, force replace format(`format')
		
	}

	* Put into one column
	foreach v of local varnames {
		if "`v'" == "var" | "`v'"=="`table'" continue
		else {
			qui set obs `=_N+1'
			qui replace var = "`v'" in `=_N'
			qui replace `table' = `v'[1] in `=_N'
		}
	}

	keep var `table'
	order var

	* Merge with dataset if specified
	if "`append'"!= "" {

		* Take care that the sorting is not messed up by the merge (and add extra joinby option for Version 8 of Stata)
		tempname sortid
		gen `sortid' = _n
		cap qui merge var using `"`using'"', sort
		if _rc !=0 {
			qui joinby var using `"`using'"', unmatched(both)
		}
		qui replace `sortid' = -1 if _merge==2
		sort `sortid', stable
		local replace replace
		drop _merge `sortid'

		* autoid option
		if "`autoid'"!="" {

			unab table_name_vars : *
			local table_name_vars : subinstr local table_name_vars "var" ""
			
			local max_id = 0
			foreach v of local table_name_vars {
				qui levelsof `v' if var=="_id", local(tmp_id) clean
				capture confirm integer number `tmp_id'
				if _rc==0 {
					if `max_id' < `tmp_id' local max_id = `tmp_id'
				}
			}
			qui replace `table' = `max_id'+1 if var=="_id"
		}

		
		* Order columns so that the more recent result are on the right
		unab colnames : *
		tokenize "`colnames'"
		while "`2'"!="" {
			macro shift
		}
		move `table' `1'

		* If number of variables has not increased, issue a warning because user probably accidently overwrote data
		qui d
		local num_vars_new = `r(k)'
		qui d using "`using'"
		if `r(k)'==`num_vars_new' di in yellow "Warning: width of using dataset has not increased. Values may have been overwritten."
		
	}

	* Apply order option, if specified
	* Known issue - if user doesn't specify a valid var, nothing happens.
	if "`order'"!=""{
		tokenize `"`order'"'
		local index = 1
		qui gen index = .
		while "`1'"!= "" {
			if "`1'"=="regvars" {				
				foreach v of local variable_names {
					qui replace index = `index' if var=="`v'"
				}		
			}
			else {
				qui count if var=="`1'"
				* Need to handle situation where user specifies, e.g., "var". We need to take care of var_coef, var_stderr, etc.
				if `r(N)' > 0 qui replace index = `index' if var=="`1'"
				else qui replace index = `index' if inlist(var,"`1'_coef","`1'_stderr","`1'_tstat","`1'_pval")
			}
			macro shift
			local index = `index'+1
		}		
		sort index, stable
		drop index
	}	

	* Restore dataset if necessary
	if "`using'"!= "" {
		`save' "`using'", `replace' `saveold_opts'
		restore	
	}
	else restore, not
	
	* Return shape of new dataset
	return clear
	qui count
	return scalar N = `r(N)'
	unab all_vars : *
	local num_vars : word count `all_vars'
	return scalar k = `num_vars'
end

